//----------------------------------------------------------------
// main.cpp
//----------------------------------------------------------------
// Copyright of John Wong 1998
//
// You are free to use this source code as long as it is for
// non-commercial use and this copyright notice is retained.
//----------------------------------------------------------------
// Revision History
//
// [JW-MAY.22.1998] v01.00  John Wong
//----------------------------------------------------------------

#include "pch.h"
#include "main.h"
#include "bitmap.h"

HINSTANCE			g_hInstance = NULL;
HWND				g_hWndMain = NULL;

static HPALETTE		g_hPalette = NULL;
static int			g_lState = 0;

LRESULT CALLBACK WindowProc(  
	HWND hwnd,		// handle to window
	UINT uMsg,		// message identifier
	WPARAM wParam,	// first message parameter
	LPARAM lParam);	// second message parameter

HPALETTE SetMainPalette(HPALETTE a_hPalette)
{
	HPALETTE hOldPalette = g_hPalette;

	g_hPalette = a_hPalette;

	if (g_hWndMain != NULL)
	{
		PostMessage(g_hWndMain, WM_QUERYNEWPALETTE, 0, 0);
	}

	return hOldPalette;
}

int WINAPI WinMain(  
	HINSTANCE a_hInstance,		// handle to current instance
	HINSTANCE a_hPrevInstance,	// handle to previous instance
	LPSTR a_lpCmdLine,			// pointer to command line
	int a_nCmdShow)				// show state of window
{
    MSG msg;     
	WNDCLASS wc;     
	
    // register the main window class
    wc.style = 0;         
	wc.lpfnWndProc = (WNDPROC) WindowProc; 
	wc.cbClsExtra = 0;         
	wc.cbWndExtra = 0; 
	wc.hInstance = a_hInstance;         
	wc.hIcon = LoadIcon((HINSTANCE) NULL, IDI_APPLICATION);         
	wc.hCursor = LoadCursor((HINSTANCE) NULL, IDC_ARROW);         
	wc.hbrBackground = (HBRUSH) GetStockObject(WHITE_BRUSH); 
	wc.lpszMenuName =  NULL; 
	wc.lpszClassName = "MainWndClass";          
	if (!RegisterClass(&wc)) 
	{
		return FALSE;     
	}
	
    g_hInstance = a_hInstance;  // save instance handle      
	
	// Create the main window. 
	g_hWndMain = CreateWindow(
		"MainWndClass", 
		"Sample 3", 
        WS_OVERLAPPEDWINDOW, 
		CW_USEDEFAULT, 
		CW_USEDEFAULT, 
        CW_USEDEFAULT, 
		CW_USEDEFAULT, 
		(HWND) NULL, 
        (HMENU) NULL, 
		g_hInstance, 
		(LPVOID) NULL);  
	
    // If the main window cannot be created, terminate     
	// the application.  
    if (!g_hWndMain)         
	{
		return FALSE;
	}
    
	// Show the window and paint its contents.  
    ShowWindow(g_hWndMain, a_nCmdShow);     
	UpdateWindow(g_hWndMain);  
	
    // Start the message loop.      
	while (GetMessage(&msg, (HWND) NULL, 0, 0)) 
    {         
		TranslateMessage(&msg);         
		DispatchMessage(&msg);     
	}  
    
	// Return the exit code to Windows.      
	return msg.wParam; 
}

LRESULT CALLBACK WindowProc(  
	HWND a_hwnd,		// handle to window
	UINT a_uMsg,		// message identifier
	WPARAM a_wParam,	// first message parameter
	LPARAM a_lParam)	// second message parameter
{
	static HBITMAP hBitmap = NULL;
	static HPALETTE hPalette1 = NULL;
	static HPALETTE hPalette2 = NULL;
	static CDibSection* pDib = NULL;

	switch (a_uMsg)
	{
	case WM_CREATE:
		{
			hBitmap = LoadBitmapFile(
				"logo.bmp",
				&hPalette1);
			SetMainPalette(hPalette1);

			pDib = new CDibSection();
			pDib->Load("logo.bmp", &hPalette2);
		}
		return 0;
	case WM_DESTROY:
		DeleteObject(hPalette1);
		DeleteObject(hPalette2);
		DeleteObject(hBitmap);
		PostQuitMessage(0);
		return 0;
	case WM_CHAR:
		{
			int value = a_wParam - '1';	// ascii code in wParam

			if (value >= 0 &&
				value < 2)
			{
				g_lState = value;
				InvalidateRect(a_hwnd, NULL, TRUE);
										// Invalidate the whole client area
			}
		}
		return 0;
	case WM_PALETTECHANGED:
		if ((HWND) a_wParam == a_hwnd)
		{
			return 0;
		}
	// fall through if not the current window changed the system palette
	case WM_QUERYNEWPALETTE:
		{
			HDC hDC = GetDC(a_hwnd);
			HPALETTE hOldPalette = SelectPalette(hDC, g_hPalette, FALSE);
			UINT nChanged = RealizePalette(hDC);
			// logical palette is already mapped. Restore the dc and release it now
			SelectPalette(hDC, hOldPalette, TRUE);
			ReleaseDC(a_hwnd, hDC);
			if (nChanged == 0)
			{
				return 0;
			}
			// make sure the screen get redraw
			InvalidateRect(a_hwnd, NULL, FALSE);
			return 1;
		}
	case WM_PAINT:
		switch (g_lState)
		{
		case 0: // bitmap sample
			{
				PAINTSTRUCT ps;
				HDC hDC;

				hDC = BeginPaint(a_hwnd, &ps);

				HPALETTE hOldPalette = SelectPalette(hDC, hPalette1, TRUE);
				RealizePalette(hDC);

				HBITMAP hOldBitmap;
				HDC hDCBitmap;

				hDCBitmap = CreateCompatibleDC(hDC);

				hOldBitmap = (HBITMAP) SelectObject(hDCBitmap, hBitmap);

				BITMAP bm;

				GetObject(hBitmap, sizeof(bm), &bm);

				BitBlt(
					hDC,
					0,
					0,
					bm.bmWidth,
					bm.bmHeight,
					hDCBitmap,
					0,
					0,
					SRCCOPY);

				// remember to do this to avoid trouble when shrinking bitmap
				SetStretchBltMode(hDC, COLORONCOLOR);
				StretchBlt(
					hDC,
					0,
					bm.bmHeight,
					bm.bmWidth / 2,
					bm.bmHeight / 2,
					hDCBitmap,
					0,
					0,
					bm.bmWidth,
					bm.bmHeight,
					SRCCOPY);

				SelectObject(hDCBitmap, hOldBitmap);
				DeleteDC(hDCBitmap);

				SelectPalette(hDC, hOldPalette, TRUE);
				EndPaint(a_hwnd, &ps);
			}
			break;
		case 1: // bitmap sample
			{
				PAINTSTRUCT ps;
				HDC hDC;

				hDC = BeginPaint(a_hwnd, &ps);

				HPALETTE hOldPalette = SelectPalette(hDC, hPalette2, TRUE);
				RealizePalette(hDC);

				BitBlt(
					hDC,
					0,
					0,
					pDib->GetWidth(),
					pDib->GetHeight(),
					pDib->GetDC(),
					0,
					0,
					SRCCOPY);

				// remember to do this to avoid trouble when shrinking bitmap

				SetStretchBltMode(hDC, COLORONCOLOR);
				StretchBlt(
					hDC,
					0,
					pDib->GetHeight(),
					pDib->GetWidth() / 2,
					pDib->GetHeight() / 2,
					pDib->GetDC(),
					0,
					0,
					pDib->GetWidth(),
					pDib->GetHeight(),
					SRCCOPY);

				SelectPalette(hDC, hOldPalette, TRUE);
				EndPaint(a_hwnd, &ps);
			}
			break;
		}
		return 0;
	default:
		return DefWindowProc(a_hwnd, a_uMsg, a_wParam, a_lParam);
	}
}

